{% import "macros/alerts.html" as alerts %}
{% import "macros/buttons.html" as buttons %}
{% import "macros/modal.html" as modal %}
{% import "macros/icons.html" as icons %}

{% extends "admin/clients/list.html" %}

{% block head_title_content %}{{ client.name }} · {{ super() }}{% endblock %}

{% set open_modal = true %}

{% block modal %}
{{
  modal.delete_modal(
    "Delete the Client \"" ~ client.name ~ "\"?",
    "If you delete this client, all the applications using this client won't be able to authenticate users anymore.",
    url_for("dashboard.clients:delete", id=client.id),
  )
}}
{% endblock %}
