{% import "macros/forms.html" as forms %}

{% extends layout %}

{% block main %}
<h1 class="text-3xl text-slate-800 font-bold mb-6">Let's create your workspace!</h1>
<form
  method="POST"
  action="{{ url_for('dashboard.workspaces:create.step1') }}"
>
  <div class="space-y-4 mb-8">
    {{ forms.form_field(form.name) }}
    {{ forms.form_csrf_token(form) }}
  </div>
  <div class="flex items-center justify-between">
    <button type="submit" class="btn bg-primary-500 hover:bg-primary-600 text-white ml-auto">Next -&gt;</button>
  </div>
</form>
{% endblock %}
