{% extends "BASE" %}

{% block preheader %}Welcome to {{ tenant.name }}! We're thrilled to have you on board.{% endblock %}

{% block main %}
<h1>Welcome!</h1>
<p>Welcome to {{ tenant.name }}! We're thrilled to have you on board.</p>
{% endblock %}
