{% extends "admin/base.html" %}

{% block head_title_content %}Error {{ status_code }}{{ super() }}{% endblock %}

{% block body %}
<main class="bg-white">

  <div class="fixed bottom-0 right-0 hidden md:block w-1/6 -z-1">
    <img src="{{ url_for('dashboard:static', path='illustrations/broken-soldier.svg') }}" alt="Fief Castle" />
  </div>

  <div class="relative flex bg-fixed bg-repeat-x bg-bottom" style="background-image: url('{{ url_for('dashboard:static', path='illustrations/grass.svg') }}')">
    <div class="w-full min-h-screen h-full flex justify-center items-center">
      <div class="text-center">
        <h1 class="text-9xl font-bold">{{ status_code }}</h1>
        <p class="text-2xl">{{ detail }}</p>
      </div>
    </div>
  </div>
</main>
{% endblock %}
