{% block head_title %}
  <title id="head-title" hx-swap-oob="true">
    {% block head_title_content %}{{ current_workspace.name }} · Fief{% endblock %}
  </title>
{% endblock %}

{% if hx_target == "main" %}
    {% block main %}{% endblock %}
{% endif %}
{% if hx_target == "aside" %}
    {% block aside %}{% endblock %}
{% endif %}
{% if hx_target == "modal" %}
    {% block modal %}{% endblock %}
{% endif %}
