{% import "macros/alerts.html" as alerts %}
{% import "macros/buttons.html" as buttons %}
{% import "macros/modal.html" as modal %}
{% import "macros/icons.html" as icons %}

{% extends "admin/oauth_providers/list.html" %}

{% block head_title_content %}{{ oauth_provider.display_name }} · {{ super() }}{% endblock %}

{% set open_modal = true %}

{% block modal %}
{{
  modal.delete_modal(
    "Delete the OAuth Provider \"" ~ oauth_provider.display_name ~ "\"?",
    "If you delete this provider, the associated OAuth accounts will be deleted as well. The associated users won't be deleted, but they will have to sign in with another method.",
    url_for("dashboard.oauth_providers:delete", id=oauth_provider.id),
  )
}}
{% endblock %}
