{% import 'macros/forms.html' as forms %}

{% extends "auth/layout.html" %}

{% block head_title %}{{ _('Reset password') }}{% endblock %}

{% block title %}{{ _('Reset password') }}{% endblock %}

{% block auth_form %}
  <form method="POST" action="{{ tenant.url_path_for(request, 'reset:reset') }}">
    <div class="space-y-4">
      {{ forms.form_field(form.password) }}
      {{ form.token }}
      {{ forms.form_csrf_token(form) }}
    </div>
    <div class="flex items-center justify-between mt-6">
      <div></div>
      <button type="submit" class="btn bg-primary hover:bg-primary-hover text-white ml-3">{{ _('Reset my password') }}</button>
    </div>
  </form>
{% endblock %}
