{% import "macros/icons.html" as icons %}

{% extends layout %}

{% block head_title_content %}Email templates · {{ super() }}{% endblock %}

{% block main %}
<div class="sm:flex sm:justify-between sm:items-center mb-8">

  <div class="mb-4 sm:mb-0">
    <h1 class="text-2xl md:text-3xl text-slate-800 font-bold">Email templates</h1>
  </div>

</div>
{% include "admin/email_templates/table.html" %}
{% endblock %}
