{% import "macros/alerts.html" as alerts %}
{% import "macros/buttons.html" as buttons %}
{% import "macros/modal.html" as modal %}
{% import "macros/icons.html" as icons %}

{% extends "admin/tenants/list.html" %}

{% block head_title_content %}{{ tenant.name }} · {{ super() }}{% endblock %}

{% set open_modal = true %}

{% block modal %}
{% call modal.delete_modal_core(
    "Delete the Tenant \"" ~ tenant.name ~ "\"?",
    url_for("dashboard.tenants:delete", id=tenant.id),
  )
%}
  <p class="text-justify">If you delete this tenant, all the associated clients and users will be deleted as well.</p>
  <ul class="text-red-500 list-disc list-inside	mt-4">
    <li>{{ users_count }} users will be <strong>deleted</strong>.</li>
    <li>{{ clients_count }} clients will be <strong>deleted</strong>.</li>
  </ul>
{% endcall %}
{% endblock %}
