{% extends "BASE" %}

{% block preheader %}Use this link to reset your password. This link is only valid for 1 hour.{% endblock %}

{% block main %}
  <h1>Reset your password</h1>
  <p>You recently requested to reset your password for your {{ tenant.name }} account. Use the button below to reset it. <strong>This password reset link is only valid for the next hour.</strong></p>
  <table class="body-action" align="center" width="100%" cellpadding="0" cellspacing="0" role="presentation">
    <tr>
      <td align="center">
        <!-- Border based button
  https://litmus.com/blog/a-guide-to-bulletproof-buttons-in-email-design -->
        <table width="100%" border="0" cellspacing="0" cellpadding="0" role="presentation">
          <tr>
            <td align="center">
              <a href="{{reset_url}}" class="f-fallback button button--green" target="_blank" rel="noopener noreferrer">Reset your password</a>
            </td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
{% endblock %}
