{% import "macros/alerts.html" as alerts %}
{% import "macros/forms.html" as forms %}
{% import "macros/icons.html" as icons %}

{% extends layout %}

{% block main %}
<h1 class="text-3xl text-slate-800 font-bold mb-6">Everything is ready!</h1>
<form
  method="POST"
  action="{{ url_for('dashboard.workspaces:create.step4') }}"
>
  <div class="space-y-4 mb-8">
    {{ forms.form_csrf_token(form) }}
    {% if error %}
      {% call alerts.error() %}
        {{ error }}
      {% endcall %}
    {% endif %}
  </div>
  <div class="flex items-center justify-between">
    <a href="{{ url_for('dashboard.workspaces:create.step3') }}" class="text-sm underline hover:no-underline">&lt;- Back</a>
    <button type="submit" class="btn bg-primary-500 hover:bg-primary-600 text-white ml-auto">Create workspace -&gt;</button>
  </div>
</form>
{% endblock %}
