"""Add database_ssl_mode to Workspace

Revision ID: 4c2bb65882d7
Revises: 696e383aabf2
Create Date: 2022-06-09 09:57:28.046792

"""
import sqlalchemy as sa
from alembic import op

import fief

# revision identifiers, used by Alembic.
revision = "4c2bb65882d7"
down_revision = "696e383aabf2"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "workspaces", sa.Column("database_ssl_mode", sa.Text(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workspaces", "database_ssl_mode")
    # ### end Alembic commands ###
