{% import "macros/icons.html" as icons %}

{% macro warning() %}
<div class="px-4 py-2 rounded-sm text-sm bg-yellow-100 border-yellow-200 text-yellow-600">
  <div class="flex w-full justify-between items-start">
    <div class="flex">
      {{ icons.exclamation_circle('w-4 h-4 shrink-0 fill-current opacity-80 mt-[3px] mr-3') }}
      <div>{{ caller() }}</div>
    </div>
  </div>
</div>
{% endmacro %}

{% macro error() %}
<div class="px-4 py-2 rounded-sm text-sm bg-red-100 border border-red-200 text-red-600">
  <div class="flex w-full justify-between items-start">
    <div class="flex">
      {{ icons.exclamation_circle('w-4 h-4 shrink-0 fill-current opacity-80 mt-[3px] mr-3') }}
      <div>{{ caller() }}</div>
    </div>
  </div>
</div>
{% endmacro %}

{% macro success() %}
<div class="px-4 py-2 rounded-sm text-sm bg-green-100 border border-green-200 text-green-600">
  <div class="flex w-full justify-between items-start">
    <div class="flex">
      {{ icons.check_circle('w-4 h-4 shrink-0 fill-current opacity-80 mt-[3px] mr-3') }}
      <div>{{ caller() }}</div>
    </div>
  </div>
</div>
{% endmacro %}
