{% import "macros/alerts.html" as alerts %}
{% import "macros/buttons.html" as buttons %}
{% import "macros/modal.html" as modal %}
{% import "macros/icons.html" as icons %}

{% extends "admin/user_fields/list.html" %}

{% block head_title_content %}{{ user_field.name }} · {{ super() }}{% endblock %}

{% set open_modal = true %}

{% block modal %}
{{
  modal.delete_modal(
    "Delete the User field \"" ~ user_field.name ~ "\"?",
    "If you delete this field, the associated values on every users will also be deleted.",
    url_for("dashboard.user_fields:delete", id=user_field.id),
  )
}}
{% endblock %}
