<!DOCTYPE html>
<html lang="en">
  <head>
    {% block head %}
      {% block head_title %}
        <title id="head-title" hx-swap-oob="true">{% block head_title_content %}{% endblock %}</title>
      {% endblock %}
      <meta name="viewport" content="width=device-width, initial-scale=1">
      <link rel="icon" href="{{ url_for('dashboard:static', path='/favicon.svg') }}" />
      <link href="{{ url_for('dashboard:static', path='/admin.css') }}" rel="stylesheet">
      <script src="https://unpkg.com/htmx.org@1.8.4"></script>
      <script src="https://unpkg.com/hyperscript.org@0.9.7"></script>
      {% block javascripts_modules %}{% endblock %}
    {% endblock %}
  </head>
  <body class="font-inter antialiased bg-gray-100 text-gray-600">
    {% block hyperscripts %}{% endblock %}
    {% block body %}{% endblock %}
  </body>
</html>
