"""Add logo_url column to Tenant

Revision ID: 31f6171a84fa
Revises: f72167c6b223
Create Date: 2023-01-06 08:26:27.744414

"""
import sqlalchemy as sa
from alembic import op

import fief

# revision identifiers, used by Alembic.
revision = "31f6171a84fa"
down_revision = "f72167c6b223"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "fief_tenants", sa.Column("logo_url", sa.String(length=512), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("fief_tenants", "logo_url")
    # ### end Alembic commands ###
