{% import "macros/icons.html" as icons %}

{% extends layout %}

{% block head_title_content %}OAuth Providers · {{ super() }}{% endblock %}

{% block main %}
<div class="sm:flex sm:justify-between sm:items-center mb-8">

  <div class="mb-4 sm:mb-0">
    <h1 class="text-2xl md:text-3xl text-slate-800 font-bold">OAuth Providers</h1>
  </div>

  <div class="grid grid-flow-col sm:auto-cols-max justify-start sm:justify-end gap-2">
    <a
      href="{{ url_for('dashboard.oauth_providers:create') }}"
      class="btn bg-primary-500 hover:bg-primary-600 text-white"
      hx-boost="true"
      hx-target="#modal"
      _="install OpenModal"
    >
      {{ icons.plus('w-4 h-4') }}
      <span class="hidden sm:block ml-2">Create OAuth Provider</span>
    </a>
  </div>

</div>
{% include "admin/oauth_providers/table.html" %}
{% endblock %}
