{% import "macros/forms.html" as forms %}

{% call(theme) forms.form_combobox_select_field_results(themes, "id", "name", "No theme is matching your query.") %}
  {{ theme.name }}
  {% if theme.default %}
  <div class="inline-flex font-medium rounded-full text-center ml-1 px-2.5 py-0.5 bg-green-100 text-green-600">
    Default
  </div>
{% endif %}
{% endcall %}
