{% import "macros/icons.html" as icons %}

{% extends layout %}

{% block head_title_content %}Webhook logs · {{ super() }}{% endblock %}

{% block main %}
<div class="sm:flex sm:justify-between sm:items-center mb-8">

  <div class="mb-4 sm:mb-0">
    <h1 class="text-2xl md:text-3xl text-slate-800 font-bold">Webhook logs</h1>
  </div>

  <div class="grid grid-flow-col sm:auto-cols-max justify-start sm:justify-end gap-2">
    <a href="{{ url_for('dashboard.webhooks:list') }}"
      class="btn border-slate-200 hover:border-slate-300 text-slate-600">
      {{ icons.arrow_left('w-4 h-4') }}
      <span class="hidden sm:block ml-2">Back</span>
    </a>
  </div>

</div>
{% include "admin/webhooks/logs/table.html" %}
{% endblock %}
