{% import "macros/buttons.html" as buttons %}
{% import "macros/icons.html" as icons %}

{% extends "admin/oauth_providers/list.html" %}

{% block head_title_content %}{{ oauth_provider.display_name }} · {{ super() }}{% endblock %}

{% set open_aside = true %}

{% block aside %}
<h2 class="text-slate-800 font-semibold text-center mb-6">{{ oauth_provider.display_name }}</h2>
<div class="mt-6">
  <ul>
    <li class="flex items-center justify-between py-3 border-b border-slate-200">
      <div class="text-sm whitespace-nowrap">ID</div>
      <div class="text-sm font-medium text-slate-800 ml-2 truncate">{{ oauth_provider.id }}</div>
      {{ buttons.clipboard(oauth_provider.id) }}
    </li>
  </ul>
</div>
<div class="mt-6">
  <a
    href="{{ url_for('dashboard.oauth_providers:update', id=oauth_provider.id) }}"
    class="btn w-full border-slate-200 hover:border-slate-300"
    hx-boost="true"
    hx-target="#modal"
    _="install OpenModal"
  >
    Edit OAuth Provider
  </a>
</div>
<div class="mt-6">
  <a
    href="{{ url_for('dashboard.oauth_providers:delete', id=oauth_provider.id) }}"
    class="btn w-full bg-red-500 hover:bg-red-600 text-white"
    hx-boost="true"
    hx-target="#modal"
    _="install OpenModal"
  >
    Delete OAuth Provider
  </a>
</div>
{% endblock %}
