{% import "macros/alerts.html" as alerts %}
{% import "macros/buttons.html" as buttons %}
{% import "macros/modal.html" as modal %}
{% import "macros/icons.html" as icons %}

{% extends "admin/roles/list.html" %}

{% block head_title_content %}{{ role.name }} · {{ super() }}{% endblock %}

{% set open_modal = true %}

{% block modal %}
{{
  modal.delete_modal(
    "Delete the Role \"" ~ role.name ~ "\"?",
    "If you delete this role, it'll be removed from all the users on which it was assigned. Make sure you don't use it anymore in your application.",
    url_for("dashboard.roles:delete", id=role.id),
  )
}}
{% endblock %}
