"""Add users_count column to Workspace

Revision ID: 3b349d4ffa69
Revises: 4c2bb65882d7
Create Date: 2022-09-29 14:51:46.714954

"""
import sqlalchemy as sa
from alembic import op

import fief

# revision identifiers, used by Alembic.
revision = "3b349d4ffa69"
down_revision = "4c2bb65882d7"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "workspaces",
        sa.Column("users_count", sa.Integer(), server_default="0", nullable=False),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workspaces", "users_count")
    # ### end Alembic commands ###
