from fief.schemas import (
    auth,
    client,
    email_template,
    generics,
    oauth_account,
    oauth_provider,
    permission,
    role,
    tenant,
    user,
    user_field,
    user_permission,
    user_role,
    webhook,
    webhook_log,
    well_known,
    workspace,
)

__all__ = [
    "workspace",
    "auth",
    "client",
    "email_template",
    "generics",
    "oauth_account",
    "oauth_provider",
    "permission",
    "role",
    "tenant",
    "user",
    "user_field",
    "user_permission",
    "user_role",
    "webhook",
    "webhook_log",
    "well_known",
]
